@@ -1,9 +1,11 @@
 import argparse
+import pkg_resources
 import re
 import sys
 import os
+from subprocess import list2cmdline
+from glob import glob
 from itertools import chain
-from pkg_resources import Distribution
 from pipes import quote
 from typing import Iterable, Set, List, Tuple, Pattern, Sequence
 from .requirements.utilities import (
@@ -30,10 +32,10 @@ def _get_requirement_string(
 ) -> str:
     requirement_string: str = os.path.abspath(directory)
     if is_installed(name) and include_extras:
-        distribution: Distribution = get_distribution(name)
+        distribution: pkg_resources.Distribution = get_distribution(name)
         if distribution.extras:
             requirement_string = (
-                f"{requirement_string}" f"[{','.join(distribution.extras)}]"
+                f"{requirement_string}[{','.join(distribution.extras)}]"
             )
     return requirement_string
 
@@ -41,6 +43,7 @@ def _get_requirement_string(
 def _iter_find_distributions(
     distribution_names: Set[str],
     directories: Iterable[str] = ("../",),
+    exclude_directories: Iterable[str] = (),
     exclude_directory_regular_expressions: Iterable[
         str
     ] = EXCLUDE_DIRECTORY_REGULAR_EXPRESSIONS,
@@ -54,6 +57,8 @@ def _iter_find_distributions(
     )
 
     def include_directory(directory: str) -> bool:
+        if os.path.abspath(directory) in exclude_directories:
+            return False
         directory_basename: str = os.path.basename(directory)
         for exclude_directory_pattern in exclude_directory_patterns:
             if exclude_directory_pattern.match(directory_basename):
@@ -89,9 +94,24 @@ def _iter_find_distributions(
     return chain(*map(iter_find_directory_distributions, directories))
 
 
+def _get_distribution_major_version(name: str) -> int:
+    version: str = ""
+    try:
+        version = pkg_resources.get_distribution("setuptools").version
+    except Exception:
+        try:
+            version = get_distribution(name).version
+        except KeyError:
+            pass
+    if version:
+        return int(version.split(".")[0])
+    return -1
+
+
 def find_and_install_distributions(
     distribution_names: Set[str],
     directories: Iterable[str] = ("../",),
+    exclude_directories: Iterable[str] = (),
     exclude_directory_regular_expressions: Iterable[
         str
     ] = EXCLUDE_DIRECTORY_REGULAR_EXPRESSIONS,
@@ -99,26 +119,64 @@ def find_and_install_distributions(
     include_extras: bool = False,
     pip_install_arguments: Sequence[str] = (),
 ) -> None:
-    requirements: Tuple[str, ...] = tuple(
+    """
+    Parameters:
+    - requirements ([str]) = ():
+      One or more requirement specifiers or configuration file paths to which
+      installation should be limited
+    - directories ([str]) = ("../",): The directories in which to search
+      for distributions to install. By default, the parent of the currently
+      directory is used.
+    - exclude ([str]): One or more distributions to pass over when searching
+      for distributable projects
+    - exclude_directories ([str]): Glob patterns indicating directories to
+      exclude. These patterns may be expressed relative to the current
+      directory.
+    - exclude_directory_regular_expressions ([str]): Directories with names
+      matching any of these patterns will be excluded. This takes into account
+      the directory *name* only, and no attempt is made to resolve relative
+      paths.
+    - dry_run (bool)
+    - include_extras (bool)
+    - pip_install_arguments ([str]): Additional arguments to pass on to
+      `pip install`
+    """
+    location: str
+    exclude_directories = set(
         map(
-            quote,
-            _iter_find_distributions(
-                distribution_names=distribution_names,
-                directories=directories,
-                exclude_directory_regular_expressions=(
-                    exclude_directory_regular_expressions
-                ),
-                include_extras=include_extras,
+            os.path.abspath,  # type: ignore
+            chain(*map(lambda location: glob(location), exclude_directories)),
+        )
+    )
+    directories = set(
+        map(
+            os.path.abspath,  # type: ignore
+            chain(*map(lambda location: glob(location), directories)),
+        )
+    )
+    requirements: Tuple[str, ...] = tuple(
+        _iter_find_distributions(
+            distribution_names=distribution_names,
+            directories=directories,
+            exclude_directory_regular_expressions=(
+                exclude_directory_regular_expressions
             ),
+            exclude_directories=exclude_directories,
+            include_extras=include_extras,
         )
     )
     if requirements:
-        command: Tuple[str, ...] = (
-            sys.executable,
-            "-m",
-            "pip",
-            "install",
-        ) + tuple(chain(*zip(("-e",) * len(requirements), requirements)))
+        command: Tuple[str, ...] = (sys.executable, "-m", "pip", "install")
+        # For setuptools version 64, we use compatibility mode to avoid
+        # issues with implicit namespace packages and mypy
+        if _get_distribution_major_version("setuptools") >= 64:
+            command += (
+                "--config-settings",
+                "editable_mode=compat",
+            )
+        command += tuple(
+            chain(*zip(("-e",) * len(requirements), requirements))
+        )
         if pip_install_arguments:
             if isinstance(pip_install_arguments, str):
                 pip_install_arguments = (pip_install_arguments,)
@@ -132,7 +190,7 @@ def find_and_install_distributions(
                 pip_install_arguments += ("-U",)
             command += pip_install_arguments
         if dry_run:
-            print(" ".join(map(quote, command)))
+            print(list2cmdline(command))
         else:
             run(command)
 
@@ -141,6 +199,7 @@ def install_editable(
     requirements: Iterable[str] = (),
     directories: Iterable[str] = ("../"),
     exclude: Iterable[str] = (),
+    exclude_directories: Iterable[str] = (),
     exclude_directory_regular_expressions: Iterable[
         str
     ] = EXCLUDE_DIRECTORY_REGULAR_EXPRESSIONS,
@@ -159,10 +218,15 @@ def install_editable(
       installation should be limited
     - directories ([str]) = ("../",): The directories in which to search
       for distributions to install. By default, the parent of the currently
-      directory is used.
+      directory is used. Glob patterns
     - exclude ([str]): One or more distributions to pass over when searching
       for distributable projects
-    - exclude_directory_regular_expressions ([str])
+    - exclude_directories ([str]): Glob patterns of directories to exclude.
+      These patterns may be expressed relative to the current directory.
+    - exclude_directory_regular_expressions ([str]): Directories with names
+      matching any of these patterns will be excluded. This takes into account
+      the directory *name* only, and no attempt is made to resolve relative
+      paths.
     - dry_run (bool)
     - include_extras (bool)
     - pip_install_arguments ([str]): Additional arguments to pass on to
@@ -178,6 +242,7 @@ def install_editable(
             required_distribution_names - set(map(normalize_name, exclude))
         ),
         directories=directories,
+        exclude_directories=exclude_directories,
         exclude_directory_regular_expressions=(
             exclude_directory_regular_expressions
         ),
@@ -239,7 +304,17 @@ def main() -> None:
         action="append",
         help="A comma-separated list of distribution names to exclude",
     )
-
+    parser.add_argument(
+        "-ed",
+        "--exclude-directory",
+        default=[],
+        type=str,
+        action="append",
+        help=(
+            "One or more glob patterns indicating directories to exclude. "
+            "This argument may be expressed as a path relative to the current."
+        ),
+    )
     parser.add_argument(
         "-edre",
         "--exclude-directory-regular-expression",
@@ -251,7 +326,9 @@ def main() -> None:
             "when searching for setup locations This argument may be passed "
             "more than once to exclude directories matching more than one "
             "regular expression. The default for this argument is "
-            "equivalent to `-edre {}`".format(
+            "equivalent to `-edre {}`. Unlike for *--exclude-directory*, "
+            "these expressions apply only to the directory *name*, and "
+            "no attempt to resolve relative paths is made.".format(
                 " -edre ".join(
                     map(quote, EXCLUDE_DIRECTORY_REGULAR_EXPRESSIONS)
                 )
@@ -286,6 +363,7 @@ def main() -> None:
         exclude_directory_regular_expressions=(
             namespace.exclude_directory_regular_expression
         ),
+        exclude_directories=namespace.exclude_directory,
         exclude=iter_parse_delimited_values(namespace.exclude),
         dry_run=namespace.dry_run,
         include_extras=namespace.include_extras,
